SharedMmioPointer

Struct SharedMmioPointer 

pub struct SharedMmioPointer<'a, T: ?Sized> { /* private fields */ }
A shared pointer to the registers of some MMIO device.

It is guaranteed to be valid but unlike UniqueMmioPointer may not be unique.

Implementations

Source

impl<T: FromBytes + IntoBytes> SharedMmioPointer<'_, T>

pub unsafe fn read_unsafe(&self) -> T

Performs an MMIO read and returns the value.

If T is exactly 1, 2, 4 or 8 bytes long then this will be a single operation. Otherwise it will be split into several, reading chunks as large as possible.

Safety

This field must be safe to perform an MMIO read from, and doing so must not cause any side-effects.

Source

impl<'a, T: ?Sized> SharedMmioPointer<'a, T>

pub const unsafe fn child<U: ?Sized>( &self, regs: NonNull<U>, ) -> SharedMmioPointer<'a, U>

Creates a new SharedMmioPointer with the same lifetime as this one.

This is used internally by the field_shared! macro and shouldn’t be called directly.

Safety

regs must be a properly aligned and valid pointer to some MMIO address space of type T, within the allocation that self points to.

pub const fn ptr(&self) -> *const T

Returns a raw const pointer to the MMIO registers.

Source

impl<T: FromBytes + IntoBytes> SharedMmioPointer<'_, ReadPure<T>>

pub fn read(&self) -> T

Performs an MMIO read of the entire T.

Source

impl<T: FromBytes + IntoBytes> SharedMmioPointer<'_, ReadPureWrite<T>>

pub fn read(&self) -> T

Performs an MMIO read of the entire T.

Source

impl<'a, T> SharedMmioPointer<'a, [T]>

pub fn split_some<const N: usize>( self, chosen: [usize; N], ) -> [UniqueMmioPointer<'a, T>; N]

Splits a UniqueMmioPointer to a slice into an array of UniqueMmioPointers, taking only the chosen indices.

Panics if chosen contains the same index more than once, or any index out of bounds.

pub const fn get(&self, index: usize) -> Option<SharedMmioPointer<'a, T>>

Returns a SharedMmioPointer to an element of this slice, or None if the index is out of bounds.

pub fn get_range( &self, range: Range<usize>, ) -> Option<SharedMmioPointer<'_, [T]>>

Returns a SharedMmioPointer to a range of elements of this slice, or None if the range is out of bounds.

pub fn iter(&self) -> SharedMmioPointerIterator<'_, T>

Returns a new iterator of the items of the slice.

pub const fn len(&self) -> usize

Returns the length of the slice.

pub const fn is_empty(&self) -> bool

Returns whether the slice is empty.

Source

impl<'a, T, const LEN: usize> SharedMmioPointer<'a, [T; LEN]>

pub fn split(self) -> [SharedMmioPointer<'a, T>; LEN]

Splits a SharedMmioPointer to an array into an array of SharedMmioPointers.

pub const fn as_slice(&self) -> SharedMmioPointer<'a, [T]>

Converts this array pointer to an equivalent slice pointer.

pub const fn get(&self, index: usize) -> Option<SharedMmioPointer<'a, T>>

Returns a SharedMmioPointer to an element of this array, or None if the index is out of bounds.

pub fn get_range( &self, range: Range<usize>, ) -> Option<SharedMmioPointer<'_, [T]>>

Returns a SharedMmioPointer to a range of elements of this array, or None if the range is out of bounds.

pub fn iter(&self) -> SharedMmioPointerIterator<'_, T>

Returns a new iterator of the items of the array.